@@ -121,7 +121,7 @@ foreach ($data['labels']['sections'] as $section_key => $section_label) {
 					->setVertical()
 					->setColumns(3)
 					->setLayoutFixed()
-					->setEnabled(!$data['readonly'])
+					->setReadonly($data['readonly'])
 			)
 		]);
 	}
@@ -320,7 +320,7 @@ $form_grid
 				'name' => 'api_methods[]',
 				'object_name' => 'api_methods',
 				'data' => $data['rules']['api'],
-				'disabled' => $data['readonly'] || !$data['rules']['api.access'],
+				'readonly' => $data['readonly'] || !$data['rules']['api.access'],
 				'popup' => [
 					'parameters' => [
 						'srctbl' => 'api_methods',
