@@ -33,15 +33,17 @@
 		_refresh_message_box: null,
 		_popup_message_box: null,
 		active_filter: null,
+		layout_mode: null,
 
 		checkbox_object: null,
 
-		init({refresh_url, refresh_data, refresh_interval, filter_options, checkbox_object, filter_set}) {
+		init({refresh_url, refresh_data, refresh_interval, filter_options, checkbox_object, filter_set, layout_mode}) {
 			this.refresh_url = new Curl(refresh_url);
 			this.refresh_data = refresh_data;
 			this.refresh_interval = refresh_interval;
 			this.checkbox_object = checkbox_object;
 			this.filter_set = filter_set;
+			this.layout_mode = layout_mode;
 
 			const url = new Curl('zabbix.php');
 			url.setArgument('action', 'latest.view.refresh');
@@ -59,14 +61,16 @@
 		},
 
 		initTabFilter(filter_options) {
-			if (!filter_options) {
-				return;
-			}
+			const filter = document.getElementById('monitoring_latest_filter');
 
 			this.refresh_counters = this.createCountersRefresh(1);
-			this.filter = new CTabFilter(document.getElementById('monitoring_latest_filter'), filter_options);
+			this.filter = new CTabFilter(filter, filter_options);
 			this.active_filter = this.filter._active_item;
 
+			if (this.layout_mode == <?= ZBX_LAYOUT_KIOSKMODE ?>) {
+				filter.style.display = 'none';
+			}
+
 			this.filter.on(TABFILTER_EVENT_URLSET, () => {
 				this.reloadPartialAndTabCounters();
 				chkbxRange.clearSelectedOnFilterChange();
@@ -444,7 +448,8 @@
 			const overlay = PopUp('item.edit', data, {
 				dialogueid: 'item-edit',
 				dialogue_class: 'modal-popup-large',
-				trigger_element: target
+				trigger_element: target,
+				prevent_navigation: true
 			});
 
 			overlay.$dialogue[0].addEventListener('dialogue.submit', this.events.elementSuccess, {once: true});
