@@ -1,21 +1,16 @@
 <?php declare(strict_types = 0);
 /*
-** Zabbix
-** Copyright (C) 2001-2023 Zabbix SIA
+** Copyright (C) 2001-2024 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 
@@ -56,7 +51,8 @@ window.sla_edit_popup = new class {
 
 		$service_tags.dynamicRows({
 			template: '#service-tag-row-tmpl',
-			rows: service_tags
+			rows: service_tags,
+			allow_empty: true
 		});
 
 		// Setup Excluded downtimes.
@@ -75,6 +71,9 @@ window.sla_edit_popup = new class {
 			});
 
 		this._update();
+
+		this.form.style.display = '';
+		this.overlay.recoverFocus();
 	}
 
 	_initTemplates() {
@@ -124,7 +123,7 @@ window.sla_edit_popup = new class {
 				row_index,
 				name: row.querySelector(`[name="excluded_downtimes[${row_index}][name]"`).value,
 				period_from: row.querySelector(`[name="excluded_downtimes[${row_index}][period_from]"`).value,
-				period_to: row.querySelector(`[name="excluded_downtimes[${row_index}][period_to]"`).value,
+				period_to: row.querySelector(`[name="excluded_downtimes[${row_index}][period_to]"`).value
 			};
 		}
 		else {
@@ -168,6 +167,8 @@ window.sla_edit_popup = new class {
 
 		this.overlay.unsetLoading();
 		this.overlay.setProperties({title, buttons});
+		this.overlay.recoverFocus();
+		this.overlay.containFocus();
 	}
 
 	delete() {
@@ -178,7 +179,7 @@ window.sla_edit_popup = new class {
 		this._post(curl.getUrl(), {slaids: [this.slaid]}, (response) => {
 			overlayDialogueDestroy(this.overlay.dialogueid);
 
-			this.dialogue.dispatchEvent(new CustomEvent('dialogue.delete', {detail: response.success}));
+			this.dialogue.dispatchEvent(new CustomEvent('dialogue.submit', {detail: response.success}));
 		});
 	}
 
