@@ -1,21 +1,16 @@
 <?php
 /*
-** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2024 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 
@@ -47,17 +42,13 @@ echo get_prepared_messages(['with_current_messages' => true]);
 if ($page['type'] == PAGE_TYPE_HTML) {
 	makeServerStatusOutput()->show();
 
-	insertPagePostJs();
-
-	require_once 'include/views/js/common.init.js.php';
-
 	if (in_array($page['type'], [PAGE_TYPE_HTML_BLOCK, PAGE_TYPE_HTML])) {
 		if (!is_null(CWebUser::$data) && isset(CWebUser::$data['debug_mode'])
 				&& CWebUser::$data['debug_mode'] == GROUP_DEBUG_MODE_ENABLED) {
 			CProfiler::getInstance()->stop();
 			CProfiler::getInstance()->show();
 
-			(new CButton(null))
+			(new CSimpleButton())
 				->addClass(ZBX_STYLE_BTN_DEBUG)
 				->show();
 		}
@@ -67,6 +58,12 @@ if ($page['type'] == PAGE_TYPE_HTML) {
 		makePageFooter()->show();
 	}
 
+	insertPagePostJs(true);
+
+	if (CWebUser::isLoggedIn()) {
+		require_once 'include/views/js/common.init.js.php';
+	}
+
 	echo '</div></body></html>';
 }
 
