@@ -1,21 +1,16 @@
 <?php
 /*
-** Zabbix
-** Copyright (C) 2001-2023 Zabbix SIA
+** Copyright (C) 2001-2024 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 
@@ -77,11 +72,15 @@ function createFontSelect(string $name): CSelect {
 	<?= (new CDiv([
 			(new CTag('h4', true, _('Map element'))),
 			(new CLink(null, CDocHelper::getUrl(CDocHelper::POPUP_MAP_ELEMENT)))
+				->addClass(ZBX_STYLE_BTN_ICON)
+				->addClass(ZBX_ICON_HELP_SMALL)
 				->setTitle(_('Help'))
-				->addClass(ZBX_STYLE_ICON_DOC_LINK)
-				->setTarget('_blank')
+				->setTarget('_blank'),
+			(new CSimpleButton())
+				->addCLass(ZBX_STYLE_BTN_OVERLAY_CLOSE)
+				->setTitle(_('Close'))
 		]))
-			->addClass(ZBX_STYLE_DASHBOARD_WIDGET_HEAD)
+			->addClass(ZBX_STYLE_OVERLAY_DIALOGUE_HEADER)
 			->setId('formDragHandler')
 			->toString()
 	?>
@@ -179,8 +178,6 @@ function createFontSelect(string $name): CSelect {
 								->setHeader(['', _('Name'), (new CColHeader(_('Action')))->addStyle('padding: 0 5px;')])
 								->setId('triggerContainer')
 								->setAttribute('style', 'width: 100%;')
-								->addClass('ui-sortable')
-								->addClass('trigger-list')
 						]))
 							->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
 							->setAttribute('style', 'min-width: '.ZBX_TEXTAREA_BIG_WIDTH.'px;')
@@ -203,8 +200,7 @@ function createFontSelect(string $name): CSelect {
 									]
 								]
 							]))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH),
-							(new CButton(null, _('Add')))
-								->addClass(ZBX_STYLE_BTN_LINK)
+							(new CButtonLink(_('Add')))
 								->setId('newSelementTriggers')
 								->addStyle('margin-top: 5px;')
 						],
@@ -220,7 +216,7 @@ function createFontSelect(string $name): CSelect {
 							->setAriaRequired(),
 						'mapSelectRow'
 					)
-					->addRow(_('Tags'),
+					->addRow(_('Problem tags'),
 						(new CDiv([
 							(new CTable())
 								->setId('selement-tags')
@@ -307,9 +303,7 @@ function createFontSelect(string $name): CSelect {
 								->setHeader([_('Name'), _('URL'), _('Action')])
 								->setId('urlContainer')
 								->setAttribute('style', 'width: 100%;'),
-							(new CButton(null, _('Add')))
-								->addClass(ZBX_STYLE_BTN_LINK)
-								->setId('newSelementUrl')
+							(new CButtonLink(_('Add')))->setId('newSelementUrl')
 						]))
 							->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
 							->setAttribute('style', 'min-width: '.ZBX_TEXTAREA_BIG_WIDTH.'px;')
@@ -317,16 +311,16 @@ function createFontSelect(string $name): CSelect {
 					->addItem([
 						(new CDiv())->addClass(ZBX_STYLE_TABLE_FORMS_TD_LEFT),
 						(new CDiv([
-							(new CButton(null, _('Apply')))
-								->addClass('element-edit-control')
-								->setId('elementApply'),
-							(new CButton(null, _('Remove')))
-								->addClass('element-edit-control')
-								->addClass(ZBX_STYLE_BTN_ALT)
-								->setId('elementRemove'),
-							(new CButton(null, _('Close')))
+							(new CSimpleButton(_('Apply')))
+								->setId('elementApply')
+								->addClass('element-edit-control'),
+							(new CSimpleButton(_('Remove')))
+								->setId('elementRemove')
 								->addClass(ZBX_STYLE_BTN_ALT)
+								->addClass('element-edit-control'),
+							(new CSimpleButton(_('Close')))
 								->setId('elementClose')
+								->addClass(ZBX_STYLE_BTN_ALT)
 						]))
 							->addClass(ZBX_STYLE_TABLE_FORMS_TD_RIGHT)
 							->addClass(ZBX_STYLE_TFOOT_BUTTONS)
@@ -340,11 +334,15 @@ function createFontSelect(string $name): CSelect {
 	<?= (new CDiv([
 			(new CTag('h4', true, _('Map shape'))),
 			(new CLink(null, CDocHelper::getUrl(CDocHelper::POPUP_MAP_SHAPE)))
+				->addClass(ZBX_STYLE_BTN_ICON)
+				->addClass(ZBX_ICON_HELP_SMALL)
 				->setTitle(_('Help'))
-				->addClass(ZBX_STYLE_ICON_DOC_LINK)
-				->setTarget('_blank')
+				->setTarget('_blank'),
+			(new CSimpleButton())
+				->addCLass(ZBX_STYLE_BTN_OVERLAY_CLOSE)
+				->setTitle(_('Close'))
 		]))
-			->addClass(ZBX_STYLE_DASHBOARD_WIDGET_HEAD)
+			->addClass(ZBX_STYLE_OVERLAY_DIALOGUE_HEADER)
 			->setId('shapeDragHandler')
 			->toString().
 		(new CForm())
@@ -475,16 +473,16 @@ function createFontSelect(string $name): CSelect {
 					->addItem([
 						(new CDiv())->addClass(ZBX_STYLE_TABLE_FORMS_TD_LEFT),
 						(new CDiv([
-							(new CButton(null, _('Apply')))
+							(new CSimpleButton(_('Apply')))
+								->setId('shapeApply')
+								->addClass('shape-edit-control'),
+							(new CSimpleButton(_('Remove')))
+								->setId('shapeRemove')
 								->addClass('shape-edit-control')
-								->setId('shapeApply'),
-							(new CButton(null, _('Remove')))
-								->addClass('shape-edit-control')
-								->addClass(ZBX_STYLE_BTN_ALT)
-								->setId('shapeRemove'),
-							(new CButton(null, _('Close')))
-								->addClass(ZBX_STYLE_BTN_ALT)
+								->addClass(ZBX_STYLE_BTN_ALT),
+							(new CSimpleButton(_('Close')))
 								->setId('shapeClose')
+								->addClass(ZBX_STYLE_BTN_ALT)
 						]))
 							->addClass(ZBX_STYLE_TABLE_FORMS_TD_RIGHT)
 							->addClass(ZBX_STYLE_TFOOT_BUTTONS)
@@ -498,11 +496,15 @@ function createFontSelect(string $name): CSelect {
 	<?= (new CDiv([
 			(new CTag('h4', true, _('Mass update shapes'))),
 			(new CLink(null, CDocHelper::getUrl(CDocHelper::POPUP_MAP_MASSUPDATE_SHAPES)))
+				->addClass(ZBX_STYLE_BTN_ICON)
+				->addClass(ZBX_ICON_HELP_SMALL)
 				->setTitle(_('Help'))
-				->addClass(ZBX_STYLE_ICON_DOC_LINK)
-				->setTarget('_blank')
+				->setTarget('_blank'),
+			(new CSimpleButton())
+				->addCLass(ZBX_STYLE_BTN_OVERLAY_CLOSE)
+				->setTitle(_('Close'))
 		]))
-			->addClass(ZBX_STYLE_DASHBOARD_WIDGET_HEAD)
+			->addClass(ZBX_STYLE_OVERLAY_DIALOGUE_HEADER)
 			->setId('massShapeDragHandler')
 			->toString().
 		(new CForm())
@@ -604,16 +606,16 @@ function createFontSelect(string $name): CSelect {
 					->addItem([
 						(new CDiv())->addClass(ZBX_STYLE_TABLE_FORMS_TD_LEFT),
 						(new CDiv([
-							(new CButton(null, _('Apply')))
+							(new CSimpleButton(_('Apply')))
+								->setId('shapeMassApply')
+								->addClass('shape-edit-control'),
+							(new CSimpleButton(_('Remove')))
+								->setId('shapeMassRemove')
 								->addClass('shape-edit-control')
-								->setId('shapeMassApply'),
-							(new CButton(null, _('Remove')))
-								->addClass('shape-edit-control')
-								->addClass(ZBX_STYLE_BTN_ALT)
-								->setId('shapeMassRemove'),
-							(new CButton(null, _('Close')))
-								->addClass(ZBX_STYLE_BTN_ALT)
+								->addClass(ZBX_STYLE_BTN_ALT),
+							(new CSimpleButton(_('Close')))
 								->setId('shapeMassClose')
+								->addClass(ZBX_STYLE_BTN_ALT)
 						]))
 							->addClass(ZBX_STYLE_TABLE_FORMS_TD_RIGHT)
 							->addClass(ZBX_STYLE_TFOOT_BUTTONS)
@@ -627,11 +629,15 @@ function createFontSelect(string $name): CSelect {
 	<?= (new CDiv([
 			(new CTag('h4', true, _('Mass update elements'))),
 			(new CLink(null, CDocHelper::getUrl(CDocHelper::POPUP_MAP_MASSUPDATE_ELEMENTS)))
+				->addClass(ZBX_STYLE_BTN_ICON)
+				->addClass(ZBX_ICON_HELP_SMALL)
 				->setTitle(_('Help'))
-				->addClass(ZBX_STYLE_ICON_DOC_LINK)
-				->setTarget('_blank')
+				->setTarget('_blank'),
+			(new CSimpleButton())
+				->addCLass(ZBX_STYLE_BTN_OVERLAY_CLOSE)
+				->setTitle(_('Close'))
 		]))
-			->addClass(ZBX_STYLE_DASHBOARD_WIDGET_HEAD)
+			->addClass(ZBX_STYLE_OVERLAY_DIALOGUE_HEADER)
 			->setId('massDragHandler')
 			->toString()
 	?>
@@ -715,16 +721,16 @@ function createFontSelect(string $name): CSelect {
 					->addItem([
 						(new CDiv())->addClass(ZBX_STYLE_TABLE_FORMS_TD_LEFT),
 						(new CDiv([
-							(new CButton(null, _('Apply')))
-								->addClass('element-edit-control')
-								->setId('massApply'),
-							(new CButton(null, _('Remove')))
-								->addClass('element-edit-control')
-								->addClass(ZBX_STYLE_BTN_ALT)
-								->setId('massRemove'),
-							(new CButton(null, _('Close')))
+							(new CSimpleButton(_('Apply')))
+								->setId('massApply')
+								->addClass('element-edit-control'),
+							(new CSimpleButton(_('Remove')))
+								->setId('massRemove')
 								->addClass(ZBX_STYLE_BTN_ALT)
+								->addClass('element-edit-control'),
+							(new CSimpleButton(_('Close')))
 								->setId('massClose')
+								->addClass(ZBX_STYLE_BTN_ALT)
 						]))
 							->addClass(ZBX_STYLE_TABLE_FORMS_TD_RIGHT)
 							->addClass(ZBX_STYLE_TFOOT_BUTTONS)
@@ -801,18 +807,16 @@ function createFontSelect(string $name): CSelect {
 								->setHeader([_('Trigger'), _('Type'), _('Color'), _('Action')])
 								->setAttribute('style', 'width: 100%;')
 								->setId('linkTriggerscontainer'),
-							(new CButton(null, _('Add')))
-								->addClass(ZBX_STYLE_BTN_LINK)
-								->onClick(
-									'return PopUp("popup.generic", '.json_encode([
-										'srctbl' => 'triggers',
-										'srcfld1' => 'triggerid',
-										'reference' => 'linktrigger',
-										'multiselect' => '1',
-										'real_hosts' => '1',
-										'with_triggers' => '1'
-									]).', {dialogue_class: "modal-popup-generic"});'
-								)
+							(new CButtonLink(_('Add')))->onClick(
+								'return PopUp("popup.generic", '.json_encode([
+									'srctbl' => 'triggers',
+									'srcfld1' => 'triggerid',
+									'reference' => 'linktrigger',
+									'multiselect' => '1',
+									'real_hosts' => '1',
+									'with_triggers' => '1'
+								]).', {dialogue_class: "modal-popup-generic"});'
+							)
 						]))
 							->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
 							->setAttribute('style', 'min-width: '.ZBX_TEXTAREA_BIG_WIDTH.'px;')
@@ -820,13 +824,14 @@ function createFontSelect(string $name): CSelect {
 					->addItem([
 						(new CDiv())->addClass(ZBX_STYLE_TABLE_FORMS_TD_LEFT),
 						(new CDiv([
-							(new CButton(null, _('Apply')))->setId('formLinkApply'),
-							(new CButton(null, _('Remove')))
-								->addClass(ZBX_STYLE_BTN_ALT)
-								->setId('formLinkRemove'),
-							(new CButton(null, _('Close')))
-								->addClass(ZBX_STYLE_BTN_ALT)
+							(new CSimpleButton(_('Apply')))
+								->setId('formLinkApply'),
+							(new CSimpleButton(_('Remove')))
+								->setId('formLinkRemove')
+								->addClass(ZBX_STYLE_BTN_ALT),
+							(new CSimpleButton(_('Close')))
 								->setId('formLinkClose')
+								->addClass(ZBX_STYLE_BTN_ALT)
 						]))
 							->addClass(ZBX_STYLE_TABLE_FORMS_TD_RIGHT)
 							->addClass(ZBX_STYLE_TFOOT_BUTTONS)
@@ -841,8 +846,7 @@ function createFontSelect(string $name): CSelect {
 			'#{toElementName}',
 			(new CCol())->addClass('element-urls'),
 			(new CCol(
-				(new CButton(null, _('Edit')))
-					->addClass(ZBX_STYLE_BTN_LINK)
+				(new CButtonLink(_('Edit')))
 					->addClass('openlink')
 					->setAttribute('data-linkid', '#{linkid}')
 			))->addClass(ZBX_STYLE_NOWRAP)
@@ -856,8 +860,7 @@ function createFontSelect(string $name): CSelect {
 			'#{toElementName}',
 			(new CCol())->addClass('element-urls'),
 			(new CCol(
-				(new CButton(null, _('Edit')))
-					->addClass(ZBX_STYLE_BTN_LINK)
+				(new CButtonLink(_('Edit')))
 					->addClass('openlink')
 					->setAttribute('data-linkid', '#{linkid}')
 			))->addClass(ZBX_STYLE_NOWRAP)
@@ -883,8 +886,7 @@ function createFontSelect(string $name): CSelect {
 			],
 			(new CColor('linktrigger_#{linktriggerid}_color', '#{color}'))->appendColorPickerJs(false),
 			(new CCol(
-				(new CButton(null, _('Remove')))
-					->addClass(ZBX_STYLE_BTN_LINK)
+				(new CButtonLink(_('Remove')))
 					->addClass('triggerRemove')
 					->setAttribute('data-linktriggerid', '#{linktriggerid}')
 			))->addClass(ZBX_STYLE_NOWRAP)
@@ -897,11 +899,10 @@ function createFontSelect(string $name): CSelect {
 <script type="text/x-jquery-tmpl" id="selementFormUrls">
 	<?= (new CRow([
 			(new CTextBox('url_#{selementurlid}_name', '#{name}'))->setWidth(ZBX_TEXTAREA_SMALL_WIDTH),
-			(new CTextBox('url_#{selementurlid}_url', '#{url}'))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH),
+			(new CTextBox('url_#{selementurlid}_url', '#{url}', false, DB::getFieldLength('sysmap_url', 'url')))
+				->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH),
 			(new CCol(
-				(new CButton(null, _('Remove')))
-					->addClass(ZBX_STYLE_BTN_LINK)
-					->onClick('jQuery("#urlrow_#{selementurlid}").remove();')
+				(new CButtonLink(_('Remove')))->onClick('jQuery("#urlrow_#{selementurlid}").remove();')
 			))->addClass(ZBX_STYLE_NOWRAP)
 		]))
 			->setId('urlrow_#{selementurlid}')
@@ -915,23 +916,18 @@ function createFontSelect(string $name): CSelect {
 
 <script type="text/x-jquery-tmpl" id="selementFormTriggers">
 	<?= (new CRow([
-			(new CCol([
-				(new CDiv())->addClass(ZBX_STYLE_DRAG_ICON),
-				(new CSpan())->addClass('ui-icon ui-icon-arrowthick-2-n-s move '.ZBX_STYLE_TD_DRAG_ICON)
-			]))->addClass(ZBX_STYLE_TD_DRAG_ICON),
+			(new CCol((new CDiv())->addClass(ZBX_STYLE_DRAG_ICON)))->addClass(ZBX_STYLE_TD_DRAG_ICON),
 			(new CCol([(new CDiv('#{name}'))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)]))
 				->addClass('#{class_name}'),
 			(new CCol([
 				(new CVar('element_id[#{triggerid}]', '#{triggerid}')),
 				(new CVar('element_name[#{triggerid}]', '#{name}')),
 				(new CVar('element_priority[#{triggerid}]', '#{priority}')),
-				(new CButton(null, _('Remove')))
-					->addClass(ZBX_STYLE_BTN_LINK)
+				(new CButtonLink(_('Remove')))
 					->addStyle('margin: 0 5px;')
 					->onClick('jQuery("#triggerrow_#{triggerid}").remove();')
 			]))->addClass(ZBX_STYLE_NOWRAP)
 		]))
-			->addClass('sortable')
 			->setId('triggerrow_#{triggerid}')
 			->toString()
 	?>
