@@ -1,26 +1,20 @@
 /*
-** Zabbix
 ** Copyright (C) 2001-2024 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 
 const EXPANDABLE_SUBFILTER_EVENT_EXPAND = 'expand';
 
-const ZBX_STYLE_ICON_WIZARD_ACTION = 'icon-wizard-action';
 const ZBX_STYLE_EXPANDED = 'expanded';
 const ZBX_STYLE_HIDDEN = 'hidden';
 
@@ -65,7 +59,7 @@ class CExpandableSubfilter extends CBaseComponent {
 
 	makeExpandButton() {
 		this.btn_expand = document.createElement('button');
-		this.btn_expand.classList.add(ZBX_STYLE_ICON_WIZARD_ACTION);
+		this.btn_expand.classList.add(ZBX_ICON_MORE);
 		this.btn_expand.addEventListener('click', () => {
 			this._target.classList.add(ZBX_STYLE_EXPANDED);
 			this.btn_expand.remove();
